//+------------------------------------------------------------------+
//|                                                 CustomPeriod.mq4 |
//|                                      Copyright Herb Spirit, Inc. |
//|                                    http://www.herbspirit.com/mql |
//+------------------------------------------------------------------+
#property copyright "Copyright Herb Spirit, Inc."
#property link      "http://www.herbspirit.com/mql"

#define HEADERSIZE	148
#define BARSIZE		44

#include <winuser32.mqh>
#include <stdlib.mqh>
#include <stderror.mqh>

//---- input parameters
extern int       ExtPeriodMultiplier=3;
extern int       TimeShift=-4;
//---- global vars
int        ExtHandle=-1;
int    i, start_pos, i_time, time0, last_fpos, periodseconds,i_period;
double d_open, d_low, d_high, d_close, d_volume, last_volume;
int    hwnd=0,cnt=0;
bool   ExpertValid;
//bool   Refresh=false;
int stdtf[]={PERIOD_M1,PERIOD_M5,PERIOD_M15,PERIOD_M30,PERIOD_H1,PERIOD_H4,
				PERIOD_D1,PERIOD_W1,PERIOD_MN1};
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
	ExpertValid=ValidateExpert();
	if(ExpertValid)
		Print(WriteHistory()," new records created");
	return(0);
}
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
   if(ExtHandle>0) 
   	FileClose(ExtHandle);
	return(0);
}
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
	if(ExpertValid)
	{
		WriteHistory();
		RefreshWindow();
		Sleep(50);
	}
	return(0);
}
//+------------------------------------------------------------------+

int WriteHistory()
{
	if(FileSize(ExtHandle)==0)
	{
//---- write history file header
	   int    version=400;
  		string c_copyright="(C)opyright Herb Spirit, Inc.";
	   int    i_digits=Digits;
  		int    i_unused[13];
	  	FileWriteInteger(ExtHandle,version,LONG_VALUE);
   	FileWriteString(ExtHandle,c_copyright,64);
	  	FileWriteString(ExtHandle,Symbol(),12);
   	FileWriteInteger(ExtHandle,i_period,LONG_VALUE);
	  	FileWriteInteger(ExtHandle,i_digits,LONG_VALUE);
   	FileWriteInteger(ExtHandle,0,LONG_VALUE);       //timesign
	  	FileWriteInteger(ExtHandle,0,LONG_VALUE);       //last_sync
   	FileWriteArray(ExtHandle,i_unused,0,13);
   }
	if(FileSize(ExtHandle)>HEADERSIZE)
	{
//---- position to last record in custom tf
		FileSeek(ExtHandle,BARSIZE*(-1),SEEK_END);
		i_time=FileReadInteger(ExtHandle);
		FileSeek(ExtHandle,-4,SEEK_CUR);
		
		FileFlush(ExtHandle);
	}
	else
	{
		FileSeek(ExtHandle,0,SEEK_END);
		i_time=iTimeShift(Bars-1)/periodseconds;
		i_time*=periodseconds;
	}
//---- position to last shift in base tf
	start_pos=iBarShift(NULL,0,i_time);
	while(iTimeShift(start_pos)<i_time&&start_pos>=0)	
		start_pos--;
	if(i_time+periodseconds<iTimeShift(start_pos))
	{
		while(i_time+periodseconds<iTimeShift(start_pos))
			i_time+=periodseconds;
		FileSeek(ExtHandle,0,SEEK_END);
	}
//---- seed initial values
	d_open=Open[start_pos];
	d_low=Low[start_pos];
	d_high=High[start_pos];
	d_close=Close[start_pos];
	d_volume=Volume[start_pos];
	int records;
	for(i=start_pos-1;i>=0; i--)
	{
//---- Write to file
		if(iTimeShift(i)>=i_time+periodseconds)
		{
			FileWriteInteger(ExtHandle,i_time,LONG_VALUE);
			FileWriteDouble(ExtHandle,d_open,DOUBLE_VALUE);
			FileWriteDouble(ExtHandle,d_low,DOUBLE_VALUE);
			FileWriteDouble(ExtHandle,d_high,DOUBLE_VALUE);
			FileWriteDouble(ExtHandle,d_close,DOUBLE_VALUE);
			FileWriteDouble(ExtHandle,d_volume,DOUBLE_VALUE);
			records++;
			i_time+=periodseconds;
			while(i_time+periodseconds<iTimeShift(i))
				i_time+=periodseconds;
			d_open=Open[i];
			d_low=Low[i];
			d_high=High[i];
			d_close=Close[i];
			d_volume=Volume[i];
		}
		else
		{
			d_volume+=Volume[i];
			if(Low[i]<d_low)   
				d_low=Low[i];
			if(High[i]>d_high) 
				d_high=High[i];
			d_close=Close[i];
		}
	} 
//---- final flush
	FileWriteInteger(ExtHandle,i_time,LONG_VALUE);
	FileWriteDouble(ExtHandle,d_open,DOUBLE_VALUE);
	FileWriteDouble(ExtHandle,d_low,DOUBLE_VALUE);
	FileWriteDouble(ExtHandle,d_high,DOUBLE_VALUE);
	FileWriteDouble(ExtHandle,d_close,DOUBLE_VALUE);
	FileWriteDouble(ExtHandle,d_volume,DOUBLE_VALUE);
	records++;
	FileFlush(ExtHandle);
	return(records);
}

void RefreshWindow()
{
	hwnd=WindowHandle(Symbol(),i_period);
//---- refresh custom timeframe window 
	if(hwnd!=0)
		PostMessageA(hwnd,WM_COMMAND,33324,0);
}

bool ValidateExpert()
{
//---- Check extern parm
	if(ExtPeriodMultiplier<2)
		return(ShowError("ExtPeriodMultiplier "+ExtPeriodMultiplier+" is less than 2"));
   i_period=Period()*ExtPeriodMultiplier;
// Check period is non-standard
	int index=ArrayBsearch(stdtf,i_period);
	if(i_period==stdtf[index])
		return(ShowError("Period "+i_period+" is standard period"));
//----  
   periodseconds=i_period*60;
  	ExtHandle=FileOpenHistory(Symbol()+i_period+".hst", FILE_BIN|FILE_READ|FILE_WRITE);
   if(ExtHandle<0) 
   {
   	int err=GetLastError();
		return(ShowError(" Error opening "+Symbol()+i_period+".hst"+" ("+err+") "+ErrorDescription(err)));
	}
	if(MathAbs(TimeShift)>23)
		return(ShowError(" TimeShift "+TimeShift+" is invalid"));
	return(true);
}

int ShowError(string msg)
{
	Print("Error: ",msg);
	return(false);
}

datetime iTimeShift(int shift)
{
	return(Time[shift]+TimeShift*3600);
}